/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef AIDGE_CORE_OPERATOR_METAOPERATOR_H_
#define AIDGE_CORE_OPERATOR_METAOPERATOR_H_

#include "aidge/operator/Operator.hpp"

namespace Aidge {
class MetaOperator : public Operator {
public:
    MetaOperator()
        : Operator("MetaOp")
    {
    }

    /**
     * @brief Copy-constructor. Copy the operator attributes and its output tensor(s), but not its input tensors (the new operator has no input associated).
     * @param op Operator to copy.
     */
    MetaOperator(const MetaOperator& op)
        : Operator("MetaOp")
    {
        // cpy-ctor
    }

    /**
     * @brief Clone the operator using its copy-constructor.
     * @see Operator::MatMul_Op
     */
    std::shared_ptr<Operator> clone() const override {
        return std::make_shared<MetaOperator>(*this);
    }

    ~MetaOperator() = default;
};
}

#endif /* MetaOperator_H_ */
