/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/


#ifndef AIDGE_UTILS_H_
#define AIDGE_UTILS_H_

#include <cstdio>
#include <memory>

#ifdef NO_EXCEPTIONS
#define AIDGE_THROW_OR_ABORT(ex, ...) \
do { std::printf(__VA_ARGS__); std::abort(); } while (false)
#else
#include <stdexcept>
#define AIDGE_THROW_OR_ABORT(ex, ...) \
do { \
    int n = 128; \
    std::unique_ptr<char[]> formatted; \
    formatted.reset(new char[n]); \
    const int len = std::snprintf(formatted.get(), n, __VA_ARGS__); \
    if (len >= n) { \
        formatted.reset(new char[len + 1]); \
        std::snprintf(formatted.get(), len + 1, __VA_ARGS__); \
    }; \
    throw ex(formatted.get()); \
} while (false)
#endif

#endif //AIDGE_UTILS_H_